@@ -46,35 +46,28 @@
 #define KEY_YELP_FIXED_FONT     KEY_YELP_DIR "/fixed_font"
 
 static const gchar * const color_params[YELP_NUM_COLORS] = {
-    "yelp.color.fg",
-    "yelp.color.bg",
-    "yelp.color.anchor",
-    "yelp.color.rule",
-    "yelp.color.gray.fg",
-    "yelp.color.gray.bg",
-    "yelp.color.gray.bg.dark1",
-    "yelp.color.gray.bg.dark2",
-    "yelp.color.gray.bg.dark3",
-    "yelp.color.selected.fg",
-    "yelp.color.selected.bg",
-    "yelp.color.selected.bg.dark1",
-    "yelp.color.selected.bg.dark2",
-    "yelp.color.selected.bg.dark3",
-    "yelp.color.admon.fg",
-    "yelp.color.admon.bg",
-    "yelp.color.admon.bg.dark1",
-    "yelp.color.admon.bg.dark2",
-    "yelp.color.admon.bg.dark3"
+    "theme.color.text",
+    "theme.color.background",
+    "theme.color.text_light",
+    "theme.color.link",
+    "theme.color.link_visited",
+    "theme.color.gray_background",
+    "theme.color.gray_border",
+    "theme.color.blue_background",
+    "theme.color.blue_border",
+    "theme.color.red_background",
+    "theme.color.red_border",
+    "theme.color.yellow_background",
+    "theme.color.yellow_border"
 };
 
 static const gchar * const icon_params[YELP_NUM_ICONS] = {
-    "yelp.icon.blockquote",
-    "yelp.icon.caution",
-    "yelp.icon.important",
-    "yelp.icon.note",
-    "yelp.icon.programlisting",
-    "yelp.icon.tip",
-    "yelp.icon.warning"
+    "theme.icon.admon.bug",
+    "theme.icon.admon.caution",
+    "theme.icon.admon.important",
+    "theme.icon.admon.note",
+    "theme.icon.admon.tip",
+    "theme.icon.admon.warning"
 };
 
 static void           settings_update         (YelpSettingsType  type);
@@ -124,55 +117,6 @@ yelp_settings_init (void)
 {
     gint i;
 
-    for (i = 0; i < YELP_NUM_ICONS; i++) {
-	switch (i) {
-	case YELP_ICON_BLOCKQUOTE:
-	    /* TRANSLATORS:
-	       This is an image of the opening quote character used to watermark
-	       blockquote elements.  Different languages use different opening
-	       quote characters, so the icon name is translatable.  The name of
-	       the icon should be "yelp-watermark-blockquote-XXXX", where XXXX
-	       is the Unicode code point of the opening quote character.   For
-	       example, some languages use the double angle quotation mark, so
-	       those would use "yelp-watermark-blockquote-00AB".  However, the
-	       image is not automagically created.  Do not translate this to a
-	       value if there isn't a corresponding icon in yelp/data/icons.
-	       If you need an image created, contact the maintainers.
-
-	       Phew, now some notes on which character to use.  Languages that
-	       use guillemets (angle quotations) should use either 00AB or 00BB,
-	       depending on whether the opening quotation is the left guillemet
-	       or the right guillemet.  Languages that use inverted comma style
-	       quotations should use 201C, 201D, or 201E.  Note that single
-	       quotation marks don't make very nice watermarks.  So if you use
-	       single quotes as your primary (outer) quotation marks, you should
-	       just use the corresponding double quote watermark.
-	    */
-	    icon_names[i] = _("yelp-watermark-blockquote-201C");
-	    break;
-	case YELP_ICON_CAUTION:
-	    icon_names[i] = "yelp-icon-caution";
-	    break;
-	case YELP_ICON_IMPORTANT:
-	    icon_names[i] = "yelp-icon-important";
-	    break;
-	case YELP_ICON_NOTE:
-	    icon_names[i] = "yelp-icon-note";
-	    break;
-	case YELP_ICON_PROGRAMLISTING:
-	    icon_names[i] = "yelp-watermark-programlisting";
-	    break;
-	case YELP_ICON_TIP:
-	    icon_names[i] = "yelp-icon-tip";
-	    break;
-	case YELP_ICON_WARNING:
-	    icon_names[i] = "yelp-icon-warning";
-	    break;
-	default:
-	    g_assert_not_reached ();
-	}
-    }
-
     gconf_client = gconf_client_get_default ();
     gconf_client_add_dir (gconf_client, KEY_GNOME_DIR,
 			  GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -215,6 +159,13 @@ yelp_settings_init (void)
 		      NULL);
 
     icon_theme = gtk_icon_theme_get_default ();
+    /* This should work with a newer GTK+ */
+    gtk_icon_theme_append_search_path (icon_theme,
+				       GDU_ICON_PATH);
+    /* But if it doesn't, this will work */
+    gtk_icon_theme_append_search_path (icon_theme,
+				       GDU_ICON_PATH"/hicolor/48x48");
+    /* And what they hey, for sentimental reasons */
     gtk_icon_theme_append_search_path (icon_theme,
 				       DATADIR "/yelp/icons");
     g_signal_connect (icon_theme,
@@ -222,6 +173,31 @@ yelp_settings_init (void)
 		      (GCallback) icon_theme_changed,
 		      NULL);
 
+    for (i = 0; i < YELP_NUM_ICONS; i++) {
+	switch (i) {
+	case YELP_ICON_ADMON_BUG:
+	    icon_names[i] = "admon-bug";
+	    break;
+	case YELP_ICON_ADMON_CAUTION:
+	    icon_names[i] = "admon-caution";
+	    break;
+	case YELP_ICON_ADMON_IMPORTANT:
+	    icon_names[i] = "admon-important";
+	    break;
+	case YELP_ICON_ADMON_NOTE:
+	    icon_names[i] = "admon-note";
+	    break;
+	case YELP_ICON_ADMON_TIP:
+	    icon_names[i] = "admon-tip";
+	    break;
+	case YELP_ICON_ADMON_WARNING:
+	    icon_names[i] = "admon-warning";
+	    break;
+	default:
+	    g_assert_not_reached ();
+	}
+    }
+
     settings_update (YELP_SETTINGS_INFO_ALL);
 }
 
@@ -352,7 +328,7 @@ yelp_settings_get_icon (YelpIconType icon)
 
     info = gtk_icon_theme_lookup_icon (icon_theme,
 				       icon_names[icon],
-				       36, 0);
+				       48, 0);
     return info;
 }
 
@@ -611,6 +587,126 @@ icon_theme_changed (GtkIconTheme *theme, gpointer user_data)
     settings_update (YELP_SETTINGS_INFO_ICONS);
 }
 
+static void
+rgb_to_hls (gdouble r, gdouble g, gdouble b, gdouble *h, gdouble *l, gdouble *s)
+{
+    gdouble min, max, delta;
+
+    if (r > g) {
+	if (r > b)
+	    max = r;
+	else
+	    max = b;
+	if (g < b)
+	    min = g;
+	else
+	    min = b;
+    }
+    else {
+	if (g > b)
+	    max = g;
+	else
+	    max = b;
+	if (r < b)
+	    min = r;
+	else
+	    min = b;
+    }
+
+    *l = (max + min) / 2;
+    *s = 0;
+    *h = 0;
+
+    if (max != min) {
+	if (*l <= 0.5)
+	    *s = (max - min) / (max + min);
+	else
+	    *s = (max - min) / (2 - max - min);
+
+	delta = max - min;
+	if (r == max)
+	    *h = (g - b) / delta;
+	else if (g == max)
+	    *h = 2 + (b - r) / delta;
+	else if (b == max)
+	    *h = 4 + (r - g) / delta;
+
+	*h *= 60;
+	if (*h < 0.0)
+	    *h += 360;
+    }
+}
+
+static void
+hls_to_hex (gdouble h, gdouble l, gdouble s, YelpColorType color)
+{
+    gdouble hue;
+    gdouble m1, m2;
+    gdouble r, g, b;
+    guint8 red, green, blue;
+
+    if (l <= 0.5)
+	m2 = l * (1 + s);
+    else
+	m2 = l + s - l * s;
+    m1 = 2 * l - m2;
+
+    if (s == 0) {
+	r = g = b = l;
+    }
+    else {
+	hue = h + 120;
+	while (hue > 360)
+	    hue -= 360;
+	while (hue < 0)
+	    hue += 360;
+
+	if (hue < 60)
+	    r = m1 + (m2 - m1) * hue / 60;
+	else if (hue < 180)
+	    r = m2;
+	else if (hue < 240)
+	    r = m1 + (m2 - m1) * (240 - hue) / 60;
+	else
+	    r = m1;
+
+	hue = h;
+	while (hue > 360)
+	    hue -= 360;
+	while (hue < 0)
+	    hue += 360;
+
+	if (hue < 60)
+	    g = m1 + (m2 - m1) * hue / 60;
+	else if (hue < 180)
+	    g = m2;
+	else if (hue < 240)
+	    g = m1 + (m2 - m1) * (240 - hue) / 60;
+	else
+	    g = m1;
+
+	hue = h - 120;
+	while (hue > 360)
+	    hue -= 360;
+	while (hue < 0)
+	    hue += 360;
+
+	if (hue < 60)
+	    b = m1 + (m2 - m1) * hue / 60;
+	else if (hue < 180)
+	    b = m2;
+	else if (hue < 240)
+	    b = m1 + (m2 - m1) * (240 - hue) / 60;
+	else
+	    b = m1;
+    }
+
+    red = r * 255;
+    green = g * 255;
+    blue = b * 255;
+    g_snprintf (colors[color], 8, "#%02X%02X%02X", red, green, blue);
+}
+
 static void
 settings_update (YelpSettingsType type)
 {
@@ -618,8 +714,9 @@ settings_update (YelpSettingsType type)
     GtkStyle  *style;
     GdkColor  *color;
     GdkColor   blue = { 0, 0x1E1E, 0x3E3E, 0xE7E7 };
-    guint8     max_text, max_base;
-    guint16    rval, gval, bval;
+    gdouble    base_h, base_l, base_s;
+    gdouble    text_h, text_l, text_s;
+    /* guint16    rval, gval, bval; */
     gint i;
 
     debug_print (DB_FUNCTION, "entering\n");
@@ -633,12 +730,14 @@ settings_update (YelpSettingsType type)
 	else
 	    style = gtk_style_new ();
 
-	max_text = MAX(style->text[GTK_STATE_NORMAL].red,
-		       MAX(style->text[GTK_STATE_NORMAL].green,
-			   style->text[GTK_STATE_NORMAL].blue   )) >> 8;
-	max_base = MAX(style->base[GTK_STATE_NORMAL].red,
-		       MAX(style->base[GTK_STATE_NORMAL].green,
-			   style->base[GTK_STATE_NORMAL].blue   )) >> 8;
+	rgb_to_hls (style->text[GTK_STATE_NORMAL].red / 65535.0,
+		    style->text[GTK_STATE_NORMAL].green / 65535.0,
+		    style->text[GTK_STATE_NORMAL].blue / 65535.0,
+		    &text_h, &text_l, &text_s);
+	rgb_to_hls (style->bg[GTK_STATE_NORMAL].red / 65535.0,
+		    style->bg[GTK_STATE_NORMAL].green / 65535.0,
+		    style->bg[GTK_STATE_NORMAL].blue / 65535.0,
+		    &base_h, &base_l, &base_s);
 
 	/* YELP_COLOR_FG */
 	g_snprintf (colors[YELP_COLOR_FG], 8,
@@ -665,67 +764,61 @@ settings_update (YelpSettingsType type)
 		    color->blue >> 8);
 	if (color != &blue)
 	    gdk_color_free (color);
-	gtk_object_sink (GTK_OBJECT (widget));
-
-	/* YELP_COLOR_RULE */
-	g_snprintf (colors[YELP_COLOR_RULE], 8,
-		    "#%02X%02X%02X",
-		    ((style->base[GTK_STATE_NORMAL].red >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].red >> 8) ) / 2,
-		    ((style->base[GTK_STATE_NORMAL].green >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].green >> 8) ) / 2,
-		    ((style->base[GTK_STATE_NORMAL].blue >> 8) + 
-		     (style->bg[GTK_STATE_NORMAL].blue >> 8) ) / 2);
-
-	/* YELP_COLOR_GRAY_BG */
-	for (i = 0; i < 4; i++) {
-	    rval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].red   >> 8) +
-		    i * max_text) / 4;
-	    gval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].green >> 8) +
-		    i * max_text) / 4;
-	    bval = ((4 - i) * (style->bg[GTK_STATE_NORMAL].blue  >> 8) +
-		    i * max_text) / 4;
-	    g_snprintf (colors[YELP_COLOR_GRAY_BG + i], 8,
-			"#%02X%02X%02X", rval, gval, bval);
+	
+	color = NULL;
+	gtk_widget_style_get (widget, "visited-link-color", &color, NULL);
+	if (color) {
+	    g_snprintf (colors[YELP_COLOR_ANCHOR], 8,
+			"#%02X%02X%02X",
+			color->red >> 8,
+			color->green >> 8,
+			color->blue >> 8);
+	    gdk_color_free (color);
 	}
 
-	/* YELP_COLOR_GRAY_FG */
-	g_snprintf (colors[YELP_COLOR_GRAY_FG], 8, "%s",
-		    colors[YELP_COLOR_GRAY_BG_DARK3]);
+	g_object_ref_sink (widget);
+	g_object_unref (widget);
 
-	/* YELP_COLOR_SELECTED_FG */
-	g_snprintf (colors[YELP_COLOR_SELECTED_FG], 8,
-		    "#%02X%02X%02X",
-		    style->text[GTK_STATE_SELECTED].red >> 8,
-		    style->text[GTK_STATE_SELECTED].green >> 8,
-		    style->text[GTK_STATE_SELECTED].blue >> 8);
+	hls_to_hex (text_h, 
+		    text_l - ((text_l - base_l) * 0.4),
+		    text_s,
+		    YELP_COLOR_FG_LIGHT);
 
-	/* YELP_COLOR_SELECTED_BG */
-	for (i = 0; i < 4; i++) {
-	    rval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].red   >> 8) +
-		    i * max_text) / 4;
-	    gval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].green >> 8) +
-		    i * max_text) / 4;
-	    bval = ((4 - i) * (style->bg[GTK_STATE_SELECTED].blue  >> 8) +
-		    i * max_text) / 4;
-	    g_snprintf (colors[YELP_COLOR_SELECTED_BG + i], 8,
-			"#%02X%02X%02X", rval, gval, bval);
-	}
+	hls_to_hex (base_h, 
+		    base_l - ((base_l - text_l) * 0.05),
+		    base_s,
+		    YELP_COLOR_GRAY_BG);
+	hls_to_hex (base_h, 
+		    base_l - ((base_l - text_l) * 0.1),
+		    base_s,
+		    YELP_COLOR_GRAY_BORDER);
 
-	/* YELP_COLOR_ADMON_FG */
-	g_snprintf (colors[YELP_COLOR_ADMON_FG], 8, "%s",
-		    colors[YELP_COLOR_GRAY_BG_DARK3]);
+	hls_to_hex (base_h,
+		    base_l - ((base_l - text_l) * 0.05),
+		    0.6,
+		    YELP_COLOR_BLUE_BG);
+	hls_to_hex (base_h,
+		    base_l - ((base_l - text_l) * 0.1),
+		    0.6,
+		    YELP_COLOR_BLUE_BORDER);
 
-	/* YELP_COLOR_ADMON_BG */
-	for (i = 0; i < 4; i++) {
-	    gint mult = max_base + ((i * (max_base - max_text)) / 3);
-	    rval = ((255 * mult) / 255);
-	    gval = ((245 * mult) / 255);
-	    bval = ((207 * mult) / 255);
+	hls_to_hex (0,
+		    base_l - ((base_l - text_l) * 0.05),
+		    0.6,
+		    YELP_COLOR_RED_BG);
+	hls_to_hex (0,
+		    base_l - ((base_l - text_l) * 0.1),
+		    0.6,
+		    YELP_COLOR_RED_BORDER);
 
-	    g_snprintf (colors[YELP_COLOR_ADMON_BG + i], 8,
-			"#%02X%02X%02X", rval, gval, bval);
-	}
+	hls_to_hex (60,
+		    base_l - ((base_l - text_l) * 0.05),
+		    0.6,
+		    YELP_COLOR_YELLOW_BG);
+	hls_to_hex (60,
+		    base_l - ((base_l - text_l) * 0.1),
+		    0.6,
+		    YELP_COLOR_YELLOW_BORDER);
 
 	g_object_unref (G_OBJECT (style));
     }
@@ -742,7 +835,7 @@ yelp_settings_params (gchar ***params,
 {
     GtkIconInfo *icon_info;
     gchar *icon_file;
-    gint colors_i, icons_i;
+    gint colors_i , icons_i;
 
     if ((*params_i + 2 * (YELP_NUM_COLORS + YELP_NUM_ICONS)) >= *params_max) {
 	*params_max += 2 * (YELP_NUM_COLORS + YELP_NUM_ICONS);
@@ -763,7 +856,7 @@ yelp_settings_params (gchar ***params,
 	    icon_file = (gchar *) gtk_icon_info_get_filename (icon_info);
 	    if (icon_file)
 		(*params)[(*params_i)++] = g_strdup_printf ("\"%s\"", icon_file);
-	    else 
+	    else
 		(*params)[(*params_i)++] = g_strdup ("\"\"");
 	    gtk_icon_info_free (icon_info);
 	} else {
